/*  Rui Santos & Sara Santos - Random Nerd Tutorials
    THIS EXAMPLE WAS TESTED WITH THE FOLLOWING HARDWARE:
      REGULAR ESP32 Dev Board + 2.8 inch 240x320 TFT Display: https://makeradvisor.com/tools/2-8-inch-ili9341-tft-240x320/ and https://makeradvisor.com/tools/esp32-dev-board-wi-fi-bluetooth/
      SET UP INSTRUCTIONS: https://RandomNerdTutorials.com/esp32-tft/
    Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files.
    The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
    Complete project details: https://RandomNerdTutorials.com/esp32-tft-touchscreen-on-off-button-ili9341-arduino/
    Updated by Riccardo Grosso, 01/21/2025   http://www.ethicaldiy.org
*/

#include <SPI.h>

/*  Install the "TFT_eSPI" library by Bodmer to interface with the TFT Display - https://github.com/Bodmer/TFT_eSPI
    *** IMPORTANT: User_Setup.h available on the internet will probably NOT work with the examples available at Random Nerd Tutorials ***
    *** YOU MUST USE THE User_Setup.h FILE PROVIDED IN THE LINK BELOW IN ORDER TO USE THE EXAMPLES FROM RANDOM NERD TUTORIALS ***
    FULL INSTRUCTIONS AVAILABLE ON HOW CONFIGURE THE LIBRARY: https://RandomNerdTutorials.com/esp32-tft/   */
#include <TFT_eSPI.h>

// Install the "XPT2046_Touchscreen" library by Paul Stoffregen to use the Touchscreen - https://github.com/PaulStoffregen/XPT2046_Touchscreen
// Note: this library doesn't require further configuration
#include <XPT2046_Touchscreen.h>

TFT_eSPI tft = TFT_eSPI();

// Touchscreen pins
#define XPT2046_IRQ 36   // T_IRQ
#define XPT2046_MOSI 32  // T_DIN
#define XPT2046_MISO 39  // T_OUT
#define XPT2046_CLK 25   // T_CLK
#define XPT2046_CS 33    // T_CS

SPIClass touchscreenSPI = SPIClass(VSPI);
XPT2046_Touchscreen touchscreen(XPT2046_CS, XPT2046_IRQ);

#define SCREEN_WIDTH 320
#define SCREEN_HEIGHT 240
#define FONT_SIZE 2

// Button position and size
#define FRAME_X 60      //#define FRAME_X 60  inizio asse x
#define FRAME_Y 30    //#define FRAME_Y 60  inizio asse y
#define FRAME_W 200     //#define FRAME_W 200 larghezza x (bottoni)
#define FRAME_H 60     //#define FRAME_H 120 altezza y (bottone)

// Red zone size
#define REDBUTTON_X FRAME_X
#define REDBUTTON_Y FRAME_Y
#define REDBUTTON_W (FRAME_W / 2)
#define REDBUTTON_H FRAME_H

// Green zone size
#define GREENBUTTON_X (REDBUTTON_X + REDBUTTON_W)
#define GREENBUTTON_Y FRAME_Y
#define GREENBUTTON_W (FRAME_W / 2)
#define GREENBUTTON_H FRAME_H

// Red1 zone size
#define REDBUTTON1_X FRAME_X
#define REDBUTTON1_Y (FRAME_Y *3)+5
#define REDBUTTON1_W (FRAME_W / 2)
#define REDBUTTON1_H (FRAME_H)

// Green1 zone size
#define GREENBUTTON1_X (REDBUTTON_X + REDBUTTON_W)
#define GREENBUTTON1_Y (FRAME_Y *3)+5
#define GREENBUTTON1_W (FRAME_W / 2)
#define GREENBUTTON1_H (FRAME_H)

// Red1 zone size
#define REDBUTTON2_X FRAME_X
#define REDBUTTON2_Y (FRAME_Y *5)+10
#define REDBUTTON2_W (FRAME_W / 2)
#define REDBUTTON2_H (FRAME_H)

// Green1 zone size
#define GREENBUTTON2_X (REDBUTTON_X + REDBUTTON_W)
#define GREENBUTTON2_Y (FRAME_Y *5)+10
#define GREENBUTTON2_W (FRAME_W / 2)
#define GREENBUTTON2_H (FRAME_H)

// LED Pin
#define LED_BLUE 5     //Led BLUE
#define LED_RED 18   //Led RED
#define LED_GREEN  23   //Led GREEN
#define GND 19
#define C4 1915
#define CS4 1805
#define D4 1700
#define DS4 1607
#define E4 1519
#define F4 1432
#define FS4 1351
#define G4 1275
#define GS4 1204
#define A4 1136
#define AS4 1073
#define B4 1012
#define C5 955
#define CS5 901
#define D5 851
#define DS5 803
#define E5 758
#define F5 715
#define FS5 675
#define G5 637
#define GS5 601
#define A5 568
#define AS5 536
#define B5 506

#define SPEAKER 26

#define MOF_PIN 0
#define POT_PIN 34

int pot = 0;

int BassTab[]={E5,DS5,E5,DS5,E5,B4,D5,C5,A4,C4,E4,A4,B4,E4,A4,B4,C5,E5,DS5,E5,DS5,E5,B4,D5,C5,A4,C4,E4,A4,B4,E4,C5,B4,A4};

int timeTab[]={1 ,1 ,1 ,1 ,1 ,1 ,1 ,1 ,3 ,1 ,1 ,1 ,2 ,1 ,1 ,1 ,2 , 1 ,1 ,1 ,1 ,1 ,1 ,1 ,1 ,3 ,1 ,1 ,1 ,2 ,1 ,1 ,1 ,4};


// Touchscreen coordinates: (x, y) and pressure (z)
int x, y, z;

// Stores current button state
bool buttonState = false;
bool buttonState1 = false;
bool buttonState2 = false;

// Print Touchscreen info about X, Y and Pressure (Z) on the Serial Monitor
void printTouchToSerial(int touchX, int touchY, int touchZ) {
  Serial.print("X = ");
  Serial.print(touchX);
  Serial.print(" | Y = ");
  Serial.print(touchY);
  Serial.print(" | Pressure = ");
  Serial.print(touchZ);
  Serial.println();
  Serial.print("buttonstate: ");
  Serial.println(buttonState);
  Serial.print("buttonstate1: ");
  Serial.println(buttonState1);
  Serial.print("buttonstate2: ");
  Serial.println(buttonState2);
}

// Draw button frame
void drawFrame() {
  tft.drawRect(FRAME_X, FRAME_Y, FRAME_W, FRAME_H, TFT_BLACK);
}

// Draw a red button
void drawRedButton() {
  tft.fillRect(REDBUTTON_X, REDBUTTON_Y, REDBUTTON_W, REDBUTTON_H, TFT_RED);
  tft.fillRect(GREENBUTTON_X, GREENBUTTON_Y, GREENBUTTON_W, GREENBUTTON_H, TFT_WHITE);
  drawFrame();
  tft.setTextColor(TFT_BLACK);
  tft.setTextSize(FONT_SIZE);
  tft.setTextDatum(MC_DATUM);
  tft.drawString("Led ON", GREENBUTTON_X + (GREENBUTTON_W / 2), GREENBUTTON_Y + (GREENBUTTON_H / 2));
  buttonState = false;
}

// Draw a green button
void drawGreenButton() {
  tft.fillRect(GREENBUTTON_X, GREENBUTTON_Y, GREENBUTTON_W, GREENBUTTON_H, TFT_GREEN);
  tft.fillRect(REDBUTTON_X, REDBUTTON_Y, REDBUTTON_W, REDBUTTON_H, TFT_WHITE);
  drawFrame();
  tft.setTextColor(TFT_BLACK);
  tft.setTextSize(FONT_SIZE);
  tft.setTextDatum(MC_DATUM);
  tft.drawString("Led OFF", REDBUTTON_X + (REDBUTTON_W / 2) + 1, REDBUTTON_Y + (REDBUTTON_H / 2));
  buttonState = true;
}

// Draw a red button 1
void drawRedButton1() {
  tft.fillRect(REDBUTTON1_X, REDBUTTON1_Y, REDBUTTON1_W, REDBUTTON1_H, TFT_RED);
  tft.fillRect(GREENBUTTON1_X, GREENBUTTON1_Y, GREENBUTTON1_W, GREENBUTTON1_H, TFT_WHITE);
  drawFrame();
  tft.setTextColor(TFT_BLACK);
  tft.setTextSize(FONT_SIZE);
  tft.setTextDatum(MC_DATUM);
  tft.drawString("Spk ON", GREENBUTTON1_X + (GREENBUTTON1_W / 2), GREENBUTTON1_Y + (GREENBUTTON1_H / 2));
  buttonState1 = false;
}

// Draw a green button 1
void drawGreenButton1() {
  tft.fillRect(GREENBUTTON1_X, GREENBUTTON1_Y, GREENBUTTON1_W, GREENBUTTON1_H, TFT_GREEN);
  tft.fillRect(REDBUTTON1_X, REDBUTTON1_Y, REDBUTTON1_W, REDBUTTON1_H, TFT_WHITE);
  drawFrame();
  tft.setTextColor(TFT_BLACK);
  tft.setTextSize(FONT_SIZE);
  tft.setTextDatum(MC_DATUM);
  tft.drawString("Spk OFF", REDBUTTON1_X + (REDBUTTON1_W / 2) + 1, REDBUTTON1_Y + (REDBUTTON1_H / 2));
  buttonState1 = true;
}

// Draw a green button 2
void drawGreenButton2() {
  tft.fillRect(GREENBUTTON2_X, GREENBUTTON2_Y, GREENBUTTON2_W, GREENBUTTON2_H, TFT_GREEN);
  tft.fillRect(REDBUTTON2_X, REDBUTTON2_Y, REDBUTTON2_W, REDBUTTON2_H, TFT_WHITE);
  drawFrame();
  tft.setTextColor(TFT_BLACK);
  tft.setTextSize(FONT_SIZE);
  tft.setTextDatum(MC_DATUM);
  tft.drawString("Mtr OFF", REDBUTTON2_X + (REDBUTTON2_W / 2) + 1, REDBUTTON2_Y + (REDBUTTON2_H / 2));
  buttonState2 = true;
}

// Draw a red button 2
void drawRedButton2() {
  tft.fillRect(REDBUTTON2_X, REDBUTTON2_Y, REDBUTTON2_W, REDBUTTON2_H, TFT_RED);
  tft.fillRect(GREENBUTTON2_X, GREENBUTTON2_Y, GREENBUTTON2_W, GREENBUTTON2_H, TFT_WHITE);
  drawFrame();
  tft.setTextColor(TFT_BLACK);
  tft.setTextSize(FONT_SIZE);
  tft.setTextDatum(MC_DATUM);
  tft.drawString("Mtr ON", GREENBUTTON2_X + (GREENBUTTON2_W / 2), GREENBUTTON2_Y + (GREENBUTTON2_H / 2));
  buttonState2 = false;
}

void pinInit() {
    pinMode(SPEAKER,OUTPUT);
    digitalWrite(SPEAKER,LOW);
}
   void sound(uint8_t note_index){
 for(int i=0;i<100;i++){
     digitalWrite(SPEAKER,HIGH);
     delayMicroseconds(BassTab[note_index]);
     digitalWrite(SPEAKER,LOW);
     delayMicroseconds(BassTab[note_index]);
  }
 }  
 
void setup() {
  Serial.begin(115200);
  pinInit();
  // Start the SPI for the touchscreen and init the touchscreen
  touchscreenSPI.begin(XPT2046_CLK, XPT2046_MISO, XPT2046_MOSI, XPT2046_CS);
  touchscreen.begin(touchscreenSPI);
  // Set the Touchscreen rotation in landscape mode
  // Note: in some displays, the touchscreen might be upside down, so you might need to set the rotation to 3: touchscreen.setRotation(3);
  touchscreen.setRotation(1);

  // Start the tft display
  tft.init();
  // Set the TFT display rotation in landscape mode
  tft.setRotation(1);

  // Clear the screen before writing to it
  tft.fillScreen(TFT_BLACK);

  // Draw buttons 
  drawGreenButton();
  drawGreenButton1();
  drawGreenButton2();

  pinMode(MOF_PIN, OUTPUT);
  pinMode(POT_PIN, INPUT);
  digitalWrite(MOF_PIN, LOW);
  pinMode(LED_GREEN, OUTPUT);
  digitalWrite(LED_GREEN, LOW);
  pinMode(LED_RED , OUTPUT);
  digitalWrite(LED_RED , LOW);
  pinMode(LED_BLUE , OUTPUT);
  digitalWrite(LED_BLUE , LOW);
  pinMode(GND, OUTPUT);
  digitalWrite(GND, LOW);
}

void loop() {
  // Checks if Touchscreen was touched, and prints X, Y and Pressure (Z) info on the TFT display and Serial Monitor
  if (touchscreen.tirqTouched() && touchscreen.touched()) {
    // Get Touchscreen points
    TS_Point p = touchscreen.getPoint();
    // Calibrate Touchscreen points with map function to the correct width and height
    x = map(p.x, 200, 3700, 1, SCREEN_WIDTH);
    y = map(p.y, 240, 3800, 1, SCREEN_HEIGHT);
    z = p.z;

    printTouchToSerial(x, y, z);

  if (buttonState) {
      Serial.println("ON");
      if ((x > REDBUTTON_X) && (x < (REDBUTTON_X + REDBUTTON_W))) {
        if ((y > (REDBUTTON_Y)) && (y <= (REDBUTTON_Y + REDBUTTON_H))) {
          //Serial.println("\");
          drawRedButton();
          digitalWrite(LED_BLUE , LOW);
          digitalWrite(LED_GREEN, HIGH);
          delay(500); 
          digitalWrite(LED_GREEN, LOW);
          digitalWrite(LED_RED , HIGH);
          delay(500); 
          digitalWrite(LED_RED , LOW);
          digitalWrite(LED_BLUE , HIGH);
          delay(500); 
          digitalWrite(LED_BLUE , HIGH);
          digitalWrite(LED_GREEN, HIGH);
          digitalWrite(LED_RED, LOW);
          delay(500); 
          digitalWrite(LED_BLUE , LOW);
          digitalWrite(LED_GREEN, HIGH);
          digitalWrite(LED_RED, HIGH);
          delay(500); 
          digitalWrite(LED_BLUE , HIGH);
          digitalWrite(LED_GREEN, LOW);
          digitalWrite(LED_RED, HIGH);
          delay(500); 
          digitalWrite(LED_BLUE , HIGH);
          digitalWrite(LED_GREEN, HIGH);
          digitalWrite(LED_RED, HIGH);
          delay(500);            
        }
      }
    }
    else {
      Serial.println("OFF");
      if ((x > (GREENBUTTON_X)) && (x < (GREENBUTTON_X + GREENBUTTON_W))) {
        if ((y > (GREENBUTTON_Y)) && (y <= (GREENBUTTON_Y + GREENBUTTON_H))) {
          Serial.println("Green button pressed");
          drawGreenButton();
          digitalWrite(LED_GREEN, LOW);
          digitalWrite(LED_RED , LOW);          
          digitalWrite(LED_BLUE , LOW);
          
        }
       }
      } 
      
 if (buttonState1) {
      Serial.println("ON");
      if ((x > REDBUTTON1_X) && (x < (REDBUTTON1_X + REDBUTTON1_W))) {
        if ((y > (REDBUTTON1_Y)) && (y <= (REDBUTTON1_Y + REDBUTTON1_H))) {
          Serial.println("Red button 1 pressed");
          drawRedButton1();
  for(int note_index=0;note_index<34;note_index++)
  {
    sound(note_index);
    delay(50*timeTab[note_index]);
       }
      }
     }
    }
    else {
      Serial.println("OFF");
      if ((x > (GREENBUTTON1_X)) && (x < (GREENBUTTON1_X + GREENBUTTON1_W))) {
        if ((y > (GREENBUTTON1_Y)) && (y <= (GREENBUTTON1_Y + GREENBUTTON1_H))) {
          Serial.println("Green button 1 pressed");
          drawGreenButton1();
          digitalWrite(LED_RED , LOW);
        }
       } 
      }
       if (buttonState2) {
      Serial.println("ON");
      if ((x > REDBUTTON2_X) && (x < (REDBUTTON2_X + REDBUTTON2_W))) {
        if ((y > (REDBUTTON2_Y)) && (y <= (REDBUTTON2_Y + REDBUTTON2_H))) {
          Serial.println("Red button 2 pressed");
          drawRedButton2();
          pot = analogRead(POT_PIN);
          Serial.print("pot value: ");
          Serial.print(pot);
          Serial.print("  |  motor power: ");
          Serial.print(pot/40);
          Serial.println(" %");
  
  analogWrite(MOF_PIN, pot/16);
  delay(100);
        }
      }
    }
    else {
      Serial.println("OFF");
      if ((x > (GREENBUTTON2_X)) && (x < (GREENBUTTON2_X + GREENBUTTON2_W))) {
        if ((y > (GREENBUTTON2_Y)) && (y <= (GREENBUTTON2_Y + GREENBUTTON2_H))) {
          Serial.println("Green button 2 pressed");
          drawGreenButton2();
          analogWrite(MOF_PIN, 0);
        }
       } 
      }
     } 
    }     

 
